# dosutils, routines and examples to use in dos and 16 bit executables
# Copyright (C) 2012 neil foster <neil.foster@vampyre-adsl.demon.co.uk>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
.macro dma_entry_t p
concat_lbl \p,dma_entry:
concat_lbl \p,dma_disable: .byte 0           # bits to disable dma channel 
concat_lbl \p,dma_enable: .byte 0            # bits to enable dma channel 
concat_lbl \p,page: .byte 0                 # page port location 
concat_lbl \p,addr: .byte 0                 # addr port location 
concat_lbl \p,count: .byte 0                # count port location 
concat_lbl \p,single: .byte 0               # single mode port location 
concat_lbl \p,mode: .byte 0                 # mode port location 
concat_lbl \p,clear_ff: .byte 0             # clear flip-flop port location 
concat_lbl \p,write: .byte 0                 # bits for write transfer 
concat_lbl \p,read: .byte 0                  # bits for read transfer 
.endm

mkstruct dma_entry
.text

DMA1_STAT=      0x08            # read status register 
DMA1_WCMD=      0x08            # write command register 
DMA1_WREQ=      0x09            # write request register 
DMA1_SNGL=      0x0A            # write single bit register 
DMA1_MODE=      0x0B            # write mode register 
DMA1_CLRFF=     0x0C            # clear byte ptr flip/flop 
DMA1_MCLR=      0x0D            # master clear register 
DMA1_CLRM=      0x0E            # clear mask register 
DMA1_WRTALL=    0x0F            # write all mask register 
DMA2_STAT=      0xD0            # read status register 
DMA2_WCMD=      0xD0            # write command register 
DMA2_WREQ=      0xD2            # write request register 
DMA2_SNGL=      0xD4            # write single bit register 
DMA2_MODE=      0xD6            # write mode register 
DMA2_CLRFF=     0xD8            # clear byte ptr flip/flop 
DMA2_MCLR=      0xDA            # master clear register 
DMA2_CLRM=      0xDC            # clear mask register 
DMA2_WRTALL=    0xDE            # write all mask register 
DMA0_ADDR=      0x00            # chan 0 base adddress 
DMA0_CNT=       0x01            # chan 0 base count 
DMA1_ADDR=      0x02            # chan 1 base adddress 
DMA1_CNT=       0x03            # chan 1 base count 
DMA2_ADDR=      0x04            # chan 2 base adddress 
DMA2_CNT=       0x05            # chan 2 base count 
DMA3_ADDR=      0x06            # chan 3 base adddress 
DMA3_CNT=       0x07            # chan 3 base count 
DMA4_ADDR=      0xC0            # chan 4 base adddress 
DMA4_CNT=       0xC2            # chan 4 base count 
DMA5_ADDR=      0xC4            # chan 5 base adddress 
DMA5_CNT=       0xC6            # chan 5 base count 
DMA6_ADDR=      0xC8            # chan 6 base adddress 
DMA6_CNT=       0xCA            # chan 6 base count 
DMA7_ADDR=      0xCC            # chan 7 base adddress 
DMA7_CNT=       0xCE            # chan 7 base count 
DMA0_PAGE=      0x87            # chan 0 page register (refresh) 
DMA1_PAGE=      0x83            # chan 1 page register 
DMA2_PAGE=      0x81            # chan 2 page register 
DMA3_PAGE=      0x82            # chan 3 page register 
DMA4_PAGE=      0x8F            # chan 4 page register (unuseable) 
DMA5_PAGE=      0x8B            # chan 5 page register 
DMA6_PAGE=      0x89            # chan 6 page register 
DMA7_PAGE=      0x8A            # chan 7 page register 

dma_tbl:
   # channel 0 
.byte 0x04, 0x00, DMA0_PAGE, DMA0_ADDR, DMA0_CNT
.byte DMA1_SNGL, DMA1_MODE, DMA1_CLRFF, 0x48, 0x44 

   # channel 1 
.byte 0x05, 0x01, DMA1_PAGE, DMA1_ADDR, DMA1_CNT
.byte DMA1_SNGL,DMA1_MODE,DMA1_CLRFF,0x49,0x45 

   # channel 2 
.byte 0x06, 0x02, DMA2_PAGE, DMA2_ADDR, DMA2_CNT
.byte DMA1_SNGL, DMA1_MODE, DMA1_CLRFF, 0x4A, 0x46 

   # channel 3 
.byte 0x07, 0x03, DMA3_PAGE, DMA3_ADDR, DMA3_CNT
.byte DMA1_SNGL, DMA1_MODE, DMA1_CLRFF, 0x4B, 0x47 

   # channel 4 
.byte 0x04, 0x00, DMA4_PAGE, DMA4_ADDR, DMA4_CNT
.byte DMA2_SNGL, DMA2_MODE, DMA2_CLRFF, 0x48, 0x44 

   # channel 5 
.byte 0x05, 0x01, DMA5_PAGE, DMA5_ADDR, DMA5_CNT
.byte DMA2_SNGL, DMA2_MODE, DMA2_CLRFF, 0x49, 0x45 

   # channel 6 
.byte 0x06, 0x02, DMA6_PAGE, DMA6_ADDR, DMA6_CNT
.byte DMA2_SNGL, DMA2_MODE, DMA2_CLRFF, 0x4A, 0x46 

   # channel 7 
.byte 0x07, 0x03, DMA7_PAGE, DMA7_ADDR, DMA7_CNT
.byte DMA2_SNGL, DMA2_MODE, DMA2_CLRFF, 0x4B, 0x47

.macro _outb_w v,p
  mov \v,%ax
  _outb %ax,\p
  xchg %ah,%al
   _outb %ax,\p
.endm

# nicked from dma.c in allegro games programminglib
# from djgpp, do not at the moment have
# access to a 16 bit c compiler so converted it to asm
#
  #dma_start:
#  Starts the DMA controller for the specified channel, transferring
#  size bytes from addr (the block must not cross a page boundary).
#  If auto_init is set, it will use the endless repeat DMA mode.
.struct 2*sizeof_int
channel: .word 0
addr: .long 0
size: .word 0
auto_init: .word 0
input: .word 0
estruct
gsym dma_start _enter
  pusha

  lea dma_tbl,_si
  mov addr+2(_bp),%bx
  mov addr(_bp),%dx
  mov size(_bp),%cx


 if channel(_bp),b,$4,0f,b
  shr %dx
  shr %cx
0:

  dec %cx
 mov dma_entry_read(_si),%al
  if_nz input(_bp),0f,b
 mov dma_entry_write(_si),%al
0:  if_z auto_init(_bp),0f,b
  or $0x10,%al
0:
   _outb dma_entry_dma_disable(_si),dma_entry_single(_si)
   _outb %ax,dma_entry_mode(_si)
   _outb $0,dma_entry_clear_ff(_si)
   _outb_w %dx,dma_entry_addr(_si)
   _outb _bx,dma_entry_page(_si)
   _outb $0,dma_entry_clear_ff(_si)
   _outb_w _cx,dma_entry_count(_si)
   _outb dma_entry_dma_enable(_si),dma_entry_single(_si)
  popa
  leave
  ret $ret_sz
