#
#  asmlib, assembler library
#  Copyright (C) 2012 neil foster <neil.foster@vampyre-adsl.demon.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 1, or (at your option)
#  any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#/
.ifndef __dos_mac_h
__dos_mac_h=1

o_rdonly=0
o_wronly=2
o_rdwr=3
_d_dos_find_ffblk_sz=0x2b

seek_set=0
seek_cur=1
seek_end=2

_d_fa_rdonly=      1
_d_fa_hidden=      2
_d_fa_system=      4
_d_fa_label=       8
_d_fa_direc=       16
_d_fa_arch=        32
rc_addr=4*0xf1 # reserved for user use. others also do this

_d_stdin_fileno=0
_d_stdout_fileno=1
_d_stderr_fileno=2

_d_ffblk_entry_count=0x0D
_d_ffblk_cluster=0x0F
_d_ffblk_attr=0x15
_d_ffblk_time=0x16
_d_ffblk_date=0x18
_d_ffblk_size=0x1A
_d_ffblk_name=0x1E

.macro bios_wait
  mov $0x86,%ah
  int $0x15
.endm

.macro copy_extended_memory
 mov $0x87,%ah
  int $0x15
.endm

.macro bios_stuff_kbd k
.ifnb \k
  mov \k,%cx
.endif
  mov $5,%ah
  int $0x16
.endm

.macro prn_get_status prn
.ifnb \prn
   setreg \prn,%dx
.endif
  mov $0x2,%ah
  int $0x17
.endm

.macro dos_read_kbd
  mov $1,%ah
  int $0x21
.endm

.macro dos_writedl_con char
  mov \char,%dl
  dos_write_con
.endm

.macro dos_write_con
  mov $2,%ah
  int $0x21
.endm


.macro dos_cputc 
  mov $6,%ah
  int $0x21
.endm

.macro dos_getche
  mov $7,%ah
   int $0x21
.endm

.macro dos_getch
  mov $8,%ah
  int $0x21
.endm

.macro dos_print_string
  mov $9,%ah
  int $0x21
.endm


.macro get_drive_parms drive=0
.if \drive!=%dl
 movb \drive,%dl
.endif
 xor %al,%al
  xor %di,%di
 mov %di,%es
  get_drive_parameters
.endm

.macro display_char
 movb $2,%ah
 int $0x21
.endm

.macro dos_setvector vector
  mov $0x25\vector,%ax # vector number
  int $0x21
.endm

.macro dos_get_time
    mov $0x2c,%ah
  int $0x21
.endm

.macro dos_get_dta
   mov $0x2f,%ah
  int $0x21
.endm
.macro dos_get_version
  mov $0x30,%ah
  int $0x21
.endm

.macro dos_terminate_stay_resident rc,l
.ifnb \l
  mov \l,%dx
  shr $4,%dx
  inc %dx
.endif
.ifnb \rc
  setreg \rc,%al
.endif
 mov $0x31,%ah
 int $0x21
.endm

.macro dos_get_drive_pb
  mov $0x32,%ah
     int $0x21
.endm

.macro dos_set_vector
  mov $0x25,%ah
   int $0x21
.endm

.macro dos_gettime
  mov $0x2c,%ah
  int $0x21
.endm

.macro dos_getvector vector
.ifb \vector
  mov $0x35,%ah
.else
  mov $0x35\vector,%ax /* get addr ofold vector */
.endif
  int $0x21
.endm

.macro dos_mkdir
  mov $0x39,%ah
  int $0x21
.endm

.macro dos_create_file
  mov $0x3c,%ah
  int $0x21
.endm
.macro dos_open_file mode
.ifb \mode
  mov $0x3d,%ah
.else
  mov $0x3d\mode,%ax
.endif
  int $0x21
.endm

.macro dos_close_file fd,rest:vararg
.ifnb \fd
  mov \fd,%bx
.endif
  mov $0x3e,%ah
  int $0x21
.ifnb  \rest
  dos_close_file \rest
.endif
.endm

.macro dos_read_handle
  mov $0x3f,%ah
  int $0x21
.endm

.macro dos_write_handle 
  mov $0x40,%ah
  int $0x21
.endm

.macro dos_unlink
  mov $0x41,%ah
  int $0x21
.endm

.macro dos_lseek whence
.ifb \whence
   mov $0x42,%ah
.else
  mov $0x42\whence,%ax
.endif
  int $0x21
.endm
.macro dos_set_attr
 mov $0x4301,%ax
  int $0x21
.endm

.macro dos_ioctl_get_dev_info
   mov $0x4400,%ax
   int $0x21
.endm

.macro dos_set_ioctl
   mov $0x4401,%ax
  int $0x21
.endm

.macro cdrom_ioctl_output
    mov $0x4403,%ax
  int $0x21
.endm

.macro dos_get_ioctl_output_status
    mov $0x4407,%ax
  int $0x21
.endm

.macro dos_dup fd
.ifnb \fd
  mov \fd,%bx
.endif
   mov $0x45,%ah
  int $0x21
.endm

.macro dos_dup2 fd,fd_dup
.ifnb \fd
  mov \fd,%bx
.endif
.ifnb \fd_dup
  mov \fd_dup,%cx
.endif
  mov $0x46,%ah
  int $0x21
.endm

.macro dos_getcwd
  mov $0x47,%ah
   int $0x21
.endm

.macro dos_allocate_memory sz
.ifnb \sz
  mov \sz,%bx
.endif
  mov $0x48,%ah
   int $0x21
.endm

.macro dos_free seg
.ifnb \seg
   mov \seg,%ax
  mov %ax,%es
.endif
  mov $0x49,%ah
  int $0x21
.endm

.macro dos_resize_memory
  mov $0x4a,%ah
  int $0x21
.endm

.macro dos_exit rc
.ifnb \rc
  mov \rc,%al
.endif
  mov $0x4c,%ah
   int $0x21
.endm

 .macro terminate code=00
  mov $0x4c\code,%ax
int $0x21 
 .endm

.macro dos_exec_go
  mov $0x4b00,%ax
  int $0x21
.endm

.macro dos_get_rc
   mov $0x4d,%ah
  int $0x21
.endm

.macro dos_findfirst
   mov $0x4e,%ah
  int $0x21
.endm

.macro dos_findnext
   mov $0x4f,%ah
  int $0x21
.endm

.macro dos_set_psp psp
.ifnb \psp
  mov \psp,%bx
.endif
  mov $0x50,%ah
  int $0x21
.endm

.macro dos_get_current_psp
  mov $0x51,%ah
   int $0x21
.endm

.macro dos_sysvars
  mov $0x52,%ah
  int $0x21
.endm

.macro dos_memory_strategy sub_fn
.ifb \sub_fn
  mov $0x58,%ah
.else
   mov $0x58\sub_fn,%ax
.endif
  int $0x21
.endm

.macro dos_enable_drive
   mov $0x5f07,%ax
  int $0x21
.endm

.macro dos_get_psp
   mov $0x62,%ah
  int $0x21
.endm

.macro dos_flush h
.ifnb \h
  mov \h,%bx
.endif
 mov $0x68,%ah
  int $0x21
.endm

.macro bios_nanosleep n
   pushl \n
  call bios_nanosleep
.endm

.macro fdprint h,s,a
  pushal
.ifnb \a
  push _ax
  lea \a,_ax
  push _ax
.else
 push $l\@
.endif
  push \h
   call32 fdprint
.ifnb \a
  pop _ax
.else
.data
l\@: .asciz "\s"
pop_sect
.endif
  popal
.endm

.macro fdhex h,l,thing,ch=$0
  pushal
  lea l0\@,_di
 hex2asc $\l,\thing
    movw \ch,(_di)
   push $l0\@
   push \h
    call32 fdprint
  popal
.data
fdhex_l\@=\l+4
fdhex_l\@=fdhex_l\@ >> 2
l0\@: .space fdhex_l\@
  .word 0
pop_sect
.endm

.macro exhibit h,l,thing,ch='\n'
  push _di
   pushl \thing
  lea l0\@,_di
  push $\l
  call32 hex2asc
    movw $\ch,(_di)
   push $l\@
   push \h
    call32 fdprint
  pop _di
.data
l\@: .ascii "\thing=  "
l0\@: .space  \l/4
  .space 3
pop_sect
.endm

.macro print_dollar str
  lea  print_\@,%dx
  dos_print_string
.data
print_\@: .ascii "\str"
  .byte '$'
pop_sect
.endm

.macro fdputc ch,h
  push \h
  push \ch
  call fdputc
.endm

.macro fd_newline h
  pushw \h
  pushw $'\n'
    call fdputc
.endm

.macro init_args
  call pinit
  if_z %ax,l0\@
  call parse_args
l0\@:
  parse_args
.endm

.macro init_args_term
  parse_args_term
.text
.endm

.macro arg_value ch,str
  getoptvalue \ch,_get_\@_\ch,"\str"
.text 99
_get_\@_\ch:
.endm

.macro arg_item ch,var,str,gbl
  getoptitem \ch,\var,"\str"
.data 98
.ifnb \gbl
\gbl \var
.else
\var:
.endif
.byte 0
.endm

.macro arg_item_nd ch,var,str,gbl
.text 98
  getoptitem \ch,\var,"\str"
.endm

.macro fdi2asc h,l,n,ch=$0
  pushal
   lea l\@,_di
  push\l \n
  call i2asc\l
 movw \ch,(_di)
  fdprint \h,,l\@
  popal
.data
l\@:
.if \l==l
  .space 12
.elseif \l==w
  .space 7
.elseif \l==b
 .space 5
.endif
pop_sect
.endm

.macro dos_usleep d
  pushl \d
 call dos_usleep
.endm

.macro chs_readx1 d,pos,n,seg,b,t
  pushw \t
.ifb \b
   pushl \seg
.else
  xpushregs \seg,\b
  call make_linear
.endif
  pushw \n
   pushl \pos
    pushw \d
  call chs_readx1
.endm

.macro chs_readx2 d,pos,n,seg,b,t
  pushw \t
.ifb \b
   pushl \seg
.else
  xpushregs \seg,\b
  call make_linear
.endif
  pushw \n
   pushl \pos
    pushw \d
  call chs_readx2
.endm

.macro chs_writex2 d,pos,n,seg,b,t
  pushw \t
.ifb \b
   pushl \seg
.else
  xpushregs \seg,\b
  call make_linear
.endif
  pushw \n
   pushl \pos
    pushw \d
  call chs_readx2
.endm

.macro chs_writex1 d,pos,n,seg,b,t
  pushw \t
.ifb \b
   pushl \seg
.else
  xpushregs \seg,\b
  call make_linear
.endif
  pushw \n
   pushl \pos
    pushw \d
  call chs_writex1
.endm

.macro readx h,s,b,l
   pushl \l
.ifb \b
  pushl \s
.else
   push \s
  push \b
  call make_linear
.endif
   push \h
  call readx
.endm

.macro writex h,s,b,l
   pushl \l
.ifb \b
  pushl \s
.else
   push \s
  push \b
  call make_linear
.endif
   push \h
  call writex
.endm

.macro xcreate_file f,a
  pushw \a
  pushw \f
  call32 xcreate_file
.endm
.macro xopen_file f,m
  push \m
  push \f
  call32 xopen_file
.endm

.macro open_file f,m
  push \m
  push \f
  call32 open_file
.endm

.macro  lfnopen
  mov $0x716c,%ax
  int $0x21
.endm


.macro isatty h
  push \h
  call isatty
.endm

.macro do_fd h,msg,fn,l,thing,ch
  fdprint \h,"\msg"
    fd\fn \h,\l,\thing,\ch
.endm
.macro printreg reg  ch=0
  pushf
  print "\reg="
 push %dx
.if \reg!=dx
  mov %\reg,%dx
.endif
  call32 hexoutword
.if \ch!=0
  push %ax
 dos_writedl_con \ch
  pop %ax
.endif
  pop %dx
  popf
.endm

.macro printereg reg  ch=0
  print "\reg="
 push %edx
.if \reg!=edx
  mov %\reg,%edx
.endif
  call32 hexoutlong
.if \ch!=0
  push %ax
 dos_writedl_con \ch
  pop %ax
.endif
  pop %edx
.endm

.macro exit rc
  mov \rc,%al
  call32 exit
.endm
.macro exit_msg rc,msg
  puts "\msg"
    exit \rc
.endm

.macro exit_if_cf rc,msg
    jnc l\@
   exit_msg \rc,"\msg"
l\@:
.endm

.macro exit_if_nz rc,msg
    jz l\@
   exit_msg \rc,"\msg"
l\@:
.endm


.macro sleep n
  push \n
  call sleep
.endm

.macro if_kbhit goto
   bios_check_keystroke
   jnz \goto
.endm

.macro ifn_kbhit goto
   bios_check_keystroke
   jz \goto
.endm

.macro dos_write h,seg,buf,len
  push \len
  push \buf
  push \seg
  push \h
  call dos_write
.endm

.macro dos_read h,seg,buf,len
  push \len
  push \buf
  push \seg
  push \h
  call dos_read
.endm
.macro puts str
  push _si
 lea puts_l\@,_si
  call32 dos_puts
  pop _si
.data
puts_l\@: .asciz "\str"
.text
.endm

.macro cputs str
 push $l\@
  call32 cputs
.data
l\@: .asciz "\str"
.text
.endm

.macro bputs str
 push $l\@
  call32 bputs
.data
l\@: .asciz "\str"
.text
.endm

.macro getfsize fn
  push \fn
  call32 getfsize
.endm


.macro lseek fd,l,w
  push \w
  pushl \l
  push \fd
  call32 lseek
.endm


.macro getfattr fn
  push \fn
  call32 getfattr
.endm

.macro dos_bdev_t l
bdev_t \l "\l",readx,chs_readx1,lba_read
.endm

.macro dos_fat_fs_t z
fat_fs_t \z,bdev=dos_bdev_t,cl_io_fn=bdev_io
.endm

.macro volid a,b
  push \b
  push \a
 call volid
.endm

.macro get_rtc
  mov $0x2,%ah
 int $0x1a
.endm

.macro pci_find_device
  mov $0xb102,%ax
  int $0x1a
.endm


.macro dos_extender_check
   mov $0xf100,%ax
  int $0x2f
.endm

.macro pci16_read_first_ioaddr b
  push \b
  call pci16_read_first_ioaddr
.endm

.macro pci16_read_word b,n
  push \n
  push \b
  call pci16_read_word
.endm

.macro pci16_read_dword b,n
  push \n
  push \b
  call pci16_read_dword
.endm

.macro pci16_read_byte b,n
  push \n
  push \b
  call pci16_read_byte
.endm

.macro pci16_write_byte b,n,v
  push \v
   push \n
  push \b
  call pci16_write_byte
.endm

.macro pci16_write_word b,n,v
  push \v
   push \n
  push \b
  call pci16_write_word
.endm

.macro pci16_write_dword b,n,v
  pushl \v
   push \n
  push \b
  call pci16_write_dword
.endm


.macro pci16_find_device v,d,i
  push \i
  push \d
  push \v
  call pci16_find_device
.endm
.macro _cpy_ext_mem dest,src,n
  push \n
 pushl \src
  pushl \dest
  call _cpy_ext_mem
.endm

.macro cpy_ext_mem dest,src,n
  pushl \n
 pushl \src
  pushl \dest
  call cpy_ext_mem
.endm

.macro waitkey_ms ch,ms
  pushl \ms
  push \ch
 call32 waitkey_ms
.endm

.macro ext_read h,b,l
  pushl \l
  pushl \b
  push \h
  call ext_read
.endm

.macro _ext_read h,b,l,sz
  push \sz
  pushl \l
  pushl \b
  push \h
  call _ext_read
.endm

.macro pci16_scan_bus foo
  push \foo
  call pci16_scan_bus
.endm

.macro hexdump h,seg,start,len
  push \len
  push \seg
  push \start
  push \h
   call hexdump
.endm

.macro hexdumpl h,seg,start,len
  push \len
  push \seg
  push \start
  push \h
   call hexdumpl
.endm

.macro ascii_write h,seg,buf,n
  push \n
  push \seg
  push \buf
  push \h
  call ascii_write
.endm

.macro hexbytes h,seg,buf,n
  push \n
  push \seg
  push \buf
  push \h
  call hexbytes
.endm

.macro printsegoffs seg offs
.if seg!=%ds
  push %ds
  movseg \seg,%ds
.endif
.if \offs!=%dx
  push %dx
  mov \offs,%dx
.endif
  call printsegoffs
.if \offs!=%dx
  pop %dx
.endif
.if \seg!=%ds
 pop %ds
.endif
.endm

.macro do32_align a,b
  pushl \b
  pushl \a
  call32 do32_align
.endm

.macro f_far_memset seg,a,b,c
  pushl \c
  push \b
  push \seg
  push \a
  call f_far_memset
.endm

.macro f_far_memsetw seg,a,b,c
  pushl \c
  push \b
  push \seg
  push \a
  call f_far_memsetw
.endm

.macro f_far_memsetl seg,a,b,c
  pushl \c
  pushl \b
  push \seg
  push \a
  call f_far_memsetl
.endm

.macro f_lnr_memset seg,b,c
  pushl \c
  push \b
  pushl \seg
  call f_lnr_memset
.endm

.macro f_lnr_memsetw seg,b,c
  pushl \c
  push \b
  pushl \seg
  call f_lnr_memsetw
.endm

.macro f_lnr_memsetl seg,b,c
  pushl \c
  pushl \b
  pushl \seg
  call f_lnr_memsetl
.endm

.macro f_spawn p,psp=$0
  push \psp
   push \p
   call f_spawn
.endm

.macro f_far_memcpy dst_seg,d,src_seg,s,n
  pushl \n
  push \src_seg
  push \s
   push \dst_seg
  push \d
  call f_far_memcpy
.endm

.macro pci16_find_class cl,sl=$0
  push \sl
  pushl \cl
  call pci16_find_class
.endm
.macro getchx
  call32 getchx
.endm


.macro reboot fl=
.ifnb \fl
  mov $0x40,%ax
  mov %ax,%fs
  movw \fl,%fs:0x72
.endif
  ljmp $0xf000,$0xfff0
.endm

.macro malloc n
   push \n
 call malloc
.endm

.macro xmalloc n
   push \n
 call xmalloc
.endm

.macro free n
  push \n
   call free
.endm

.macro xfree n
  push \n
   call xfree
.endm

.macro pnp_get_system_nodes ep,n,sz,pnp_seg=$0,seg=%ds
  push \pnp_seg
  push \seg
  push \sz
  push \seg
  push \n
   push $0
  lcall *(\ep)
  add $0xc,%sp
.endm

.macro pnp_get_boot_devices ep,type,id,ser,lgl_id,pun,pref_res,pnp_seg=$0,seg=%ds
  push \pnp_seg
  push \seg
  push \pref_res
  push \seg
   push \pun
  push \seg
  push \lgl_id
  push \seg
  push \ser
 push \seg
  push \id
  push \type
  push $8
  lcall *(\ep)
  add $26,%sp
.endm

.macro dmi_get_info ep,rev,num,sz,st_addr,st_sz,pnp_seg=$0,seg=%ds
  push \pnp_seg
  push \seg
  push \st_sz
  push \seg
  push \st_addr
  push \seg
  push \sz
  push \seg
  push \num
  push \seg
   push \rev 
  push $0x50
  lcall *(\ep)
  add $24,%sp
.endm

.macro dmi_get_struc ep,h,b,dmi_seg=$0,pnp_seg=$0,seg=%ds
   push \pnp_seg
  push \dmi_seg
  push \seg
  push \b
  push \seg
  push \h
    push $0x51
  lcall *(\ep)
   add $0xe,%sp
.endm

.macro smb_get_gpnv_storage ep,h,sz,sz_alloc,paddr,pnp_seg=%ds=$0,seg=%ds
  push \pnp_seg
  push \seg
  push \paddr
  push  \seg
  push \sz_alloc
 push \seg
   push \sz
  push \seg
  push \h
  push $0x55
  lcall *(\ep)
   add $0x14,%sp
.endm

.macro smbios_gpnv_get_info handle,buf_sz,sz_alloc,phys_addr_mmap,seg=%ds
.ifb \phys_addr_mmap
  push $0
.else
   pushl \phys_addr_mmap
.endif
  push \seg
  push \sz_alloc
  push \seg
  push \buf_sz
  push \seg
  push \handle
  push $0x55
  call pnp_bios_
  add $18,%sp
.endm

.macro bbs_install_check ver,seg=%ds
  push \seg
  push \ver
  push $0x60
  call pnp_bios_
  add $6,%sp
.endm

.macro bbs_get_ipl_dev_count ep,n,max,sz,seg=%ds
  push \seg
  push \sz
  push \seg
  push \max
  push \seg
  push \n
  push $0x61
  lcall *(\ep)
  add $0xe,%sp
.endm

.macro bbs_get_prty_tbl ep,n,prty,tbl,seg=%ds
  push \seg
  push \tbl
  push \seg
  push \prty
  push \seg
  push \n
  push $0x62
  lcall *(\ep)
  add $0xe,%sp
.endm

.macro bbs_get_boot_first ep,ix,seg=%ds
  push \seg
  push \ix
  push $0x65
  lcall *(\ep)
  add $6,%sp
.endm

.macro relocate dest,len=$image_size,start=$_start
   mov %cs,%ax
   mov %ax,%ds
   mov  \dest,%ax
   mov %ax,%es
   mov \start,%si
  mov %si,%di
    mov \len,%cx
  cld
  rep movsb
   ljmp \dest,$relocate\@
relocate\@:
  .endm

.macro lba_read_addr32 d,start,seg,buf,n
  push \n
.ifnb \buf
 make_linear \seg,\buf
.else
  pushl \seg
.endif
  pushl \start
  push \d
  call lba_read_addr32
.endm

.macro playnote_bios_ p,d
  mov \d,%ecx
  mov \p,%ax
  call playnote_bios_
.endm

.macro f_getenv str
  push \str
   call f_getenv
.endm
.macro flip_psp psp
  push \psp
  call flip_psp
.endm

.macro exitemu rc
  mov \rc,%bx
 mov $-1,%ax
  int $0xe6
.endm

.macro far_fdprint h,seg,str
  push \str
   push \seg
  push \h
  call32 far_fdprint
.endm
.ifdef real_mode

.macro serial_write u,s,n
  push \n
  push \s
   push \u
  call serial_write
.endm
.endif

.endif
