#
#  asmlib, assembler library
#  Copyright (C) 2012 neil foster <neil.foster@vampyre-adsl.demon.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 1, or (at your option)
#  any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#/
.ifndef __ehci_mac_h
__ehci_mac_h=1

_d_ehci_hscparams=4 # structural capability params ehci 2.2.4
_d_ehci_hccparams=8 # core capability params ehci 2.2.4
_d_pci_ehci_usb_class=0xc0320

_d_ehci_usbcmd=0
_d_ehci_cmd_park=(1<<11)		# enable "park" on async qh */
_d_ehci_cmd_lreset=(1<<7)		# partial reset (no ports, etc) */
_d_ehci_cmd_iaad=(1<<6)		# "doorbell" interrupt async advance */
_d_ehci_cmd_ase=	(1<<5)		# async schedule enable */
_d_ehci_cmd_pse= 	(1<<4)		# periodic schedule enable */
_d_ehci_cmd_reset=(1<<1)		# reset hc not bus */
_d_ehci_cmd_run=	(1<<0)		# start/stop hc */

_d_ehci_usbsts=4
_d_ehci_sts_ass=	(1<<15)		# async schedule status */
_d_ehci_sts_pss=	(1<<14)		# periodic schedule status */
_d_ehci_sts_recl=(1<<13)		# reclamation */
_d_ehci_sts_halt=(1<<12)		# not running (any reason) */
_d_ehci_sts_iaa=	(1<<5)		# interrupted on async advance */
_d_ehci_sts_fatal=(1<<4)		# such as some pci access errors */
_d_ehci_sts_flr=	(1<<3)		# frame list rolled over */
_d_ehci_sts_pcd=	(1<<2)		# port change detect */
_d_ehci_sts_err=	(1<<1)		# "error" completion (overflow, ...) */
_d_ehci_sts_int=	(1<<0)		# "normal" completion (short, ...) */

_d_ehci_usbintr=8
_d_ehci_flag_cf=	(1<<0)		# true: we'll support "high speed" */
_d_ehci_portsc1=0x44
_d_ehci_usbportsc2=0x48
_d_ehci_port_wkoc_e=(1<<22)		# wake on overcurrent (enable) */
_d_ehci_port_wkdisc_e=(1<<21)		# wake on disconnect (enable) */
_d_ehci_port_wkconn_e=(1<<20)		# wake on connect (enable) */
_d_ehci_port_led_off=(0<<14)
_d_ehci_port_led_amber=(1<<14)
_d_ehci_port_led_green=(2<<14)
_d_ehci_port_led_mask=(3<<14)
_d_ehci_port_owner=(1<<13)		# true: companion hc owns this port */
_d_ehci_port_power=(1<<12)		# true: has power (see ppc) */
_d_ehci_port_reset=(1<<8)		# reset port */
_d_ehci_port_suspend=(1<<7)		# suspend port */
_d_ehci_port_resume=(1<<6)		# resume it */
_d_ehci_port_occ=(1<<5)		# over current change */
_d_ehci_port_oc=	(1<<4)		# over current active */
_d_ehci_port_pec=(1<<3)		# port enable change */
_d_ehci_port_pe=	(1<<2)		# port enable */
_d_ehci_port_csc=(1<<1)		# connect status change */
_d_ehci_port_connect=(1<<0)		# device connected */

_d_ehci_asynclistaddr=0x18
_d_ehci_configflag=0x40

_d_pci_ehci_sbrn=0x60
_d_pci_ehci_wake=0x62
.macro ehci_wait_sts_int r,i
  pushl \i
   pushl \r
  call32 ehci_wait_sts_int
.endm

.macro ehci_init_qtd qtd,sz,pid,p0
.ifnb \p0
    pushl \p0
.endif
  push \pid
   push \sz
   push \qtd
   call32 ehci_init_qtd
.endm

.macro ehci_disable_async_sched r
  pushl \r
  call32 ehci_disable_async_sched
.endm

.macro ehci_enable_async_sched r
  pushl \r
  call32 ehci_enable_async_sched
.endm

.macro ehci_init_qh qh,t,d,e,td
   push \td
  push \e
  push \d
   push \t
  push \qh
  call32 ehci_init_qh
.endm

.macro ehci16_simple_urb r,qh,qtd,pg0,sz,pid,d,ept
  push \ept
   push \d
  push \pid
   push \sz
   push \pg0
    push \qtd
    push \qh
  pushl \r
   call ehci16_simple_urb
.endm

.macro ehci_transaction r
  pushl \r
   call ehci_transaction
.endm

.macro ehci_reset_port r,p
  push \p
  pushl \r
  call32 ehci_reset_port
.endm

.macro ehci_setup_qh_qtd qh,qtd,pg0,sz,pid,d,e
  push \e
  push \d
   push \pid
   push \sz
   push \pg0
   push \qtd
   push \qh
  call32 ehci_setup_qh_qtd
.endm

.macro ehci_do_urb h,stp,ib,sz,e
 push \e
 push \sz
  push \ib
 push \stp
  push \h
  call32 ehci_do_urb
.endm

.macro _ehci_do_urb r,qh,qtd,stp,ib,sz,d,e
 push \e
   push \d
 push \sz
  push \ib
 push \stp
  push \qtd
   push \qh
  pushl \r
  call32 _ehci_do_urb
.endm

.macro ehci_read r,qh,qtd,stp,ib,sz,d,e
 push \e
   push \d
 push \sz
  push \ib
 push \stp
  push \qtd
   push \qh
  pushl \r
  call32 ehci_do_urb
.endm

.macro ehci_reset_ports r
  pushl \r
  call32 ehci_reset_ports
.endm

.macro ehci_reset_usblegctlsts sl
  push \sl
  call32 ehci_reset_usblegctlsts
.endm

.macro ehci_enable_ports r,n
  push \n
  pushl \r
  call32 ehci_enable_ports
.endm


.macro ehci16_setup_q_ptrs b,qh,qtd
   push \qtd
  push \qh
   push \b
  call ehci16_setup_q_ptrs
.endm

.macro ehci16_set_asynclistaddr r,s,o
.ifnb \o
  make_linear \s,\o
.else
  pushl \s
.endif
  push $_d_ehci_asynclistaddr
   pushl \r
  call extmem_write_dword
.endm

.macro ehci16_init sl,r,rb
   push \rb
   push \r
   push \sl
   call ehci16_init
.endm

.endif
