#
#  asmlib, assembler library
#  Copyright (C) 2012 neil foster <neil.foster@vampyre-adsl.demon.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 1, or (at your option)
#  any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#/
.ifndef __maclib_h
__maclib_h=1

.ifdef real_mode
sizeof_int=2
near_ptr_sz=2
far_ptr_sz=4
bit_shift_sz=1
_ax=%ax
_bx=%bx
_cx=%cx
_dx=%dx
_di=%di
_si=%si
_bp=%bp
_sp=%sp
.macro mov32 w:vararg
movw \w
.endm
.macro inc32 w
incw \w
.endm
.macro dec32 w
decw \w
.endm
.macro _push args:vararg
pushw \args
.endm
.macro int_t args:vararg
.word \args
.endm

.macro call32 v
  call \v
.endm
.else
sizeof_int=4
near_ptr_sz=4
far_ptr_sz=6
bit_shift_sz=2
_push=pushl
_ax=%eax
_bx=%ebx
_cx=%ecx
_dx=%edx
_di=%edi
_si=%esi
_bp=%ebp
_sp=%esp
.macro mov32 w:vararg
movl \w
.endm
.macro inc32 w
incl \w
.endm
.macro dec32 w
decl \w
.endm
.macro _push args:vararg
pushl \args
.endm
.macro int_t args:vararg
.long \args
.endm
.endif
.include "pci-mac.h"
_d_esc=0x11b
_d_f5=0x3f00
_d_f8=0x4200
_d_home=0x4700
_d_end=0x4f00
_d_up=0x4800
_d_down=0x5000

idt_entries=256
_d_sector_sz=0x200
_d_sector_shift=9
_d_int13_read=2
_d_int13_write=3
_d_dparms_sig=0x1a
.ifdef real_mode
_d_serial_time_out=0xffff
.else
_d_serial_time_out=0xffffffff
.endif

o_rdonly=0
o_wronly=1
o_rdwr=2
o_binary=4

seek_set=0
seek_cur=1
seek_end=2

_d_fa_rdonly=      1
_d_fa_hidden=      2
_d_fa_system=      4
_d_fa_label=       8
_d_fa_direc=       16
_d_fa_arch=        32

_d_stdin_fileno=0
_d_stdout_fileno=1
_d_stderr_fileno=2
_d_dt_flush_buffer_char=0x18
_d_ier=1
_d_iir=2
_d_lcr=3
_d_mcr=4
_d_lsr=5
_d_msr=6
_d_file_notfound=-1

.ifdef real_mode
.code16
stack_ptr=%sp
base_ptr=%bp
.else
stack_ptr=%esp
base_ptr=%ebp
.macro  equ_type label,type
.equ \label,\label\type
.endm
.macro equ label
equ_type \label 32
.endm
.macro call32 v,sfx=32
.ifndef _d_build_asmlib
.ifdef _d_djgpp
 call _\v\sfx
.else
.ifdef win32
 call _\v\sfx
.else
 call \v\sfx
.endif
.endif
.else
 call \v
.endif
.endm
.endif

.ifdef win32
.include "win32-mac.h"
.endif

.ifdef _d_lnx
.include "lnx-mac.h"
.endif

.macro flip_cs cs,jmp_to
  push \cs
.ifnb \jmp_to
  push \jmp_to
.else
  push $l\@
.endif
  lret
l\@:
.endm

 .macro concat_lblg a,b,c:vararg
gsym \a\b \c
.endm

.macro concat_lbll a,b,c:vararg
\a\b: \c
.endm

 .macro concat_lbl a,b,c:vararg
\a\b \c
.endm

 .macro concat_sym a,b,c:vararg
\a\b=.
 \c
.endm

.macro parm_start n=2*sizeof_int
parm_start=\n
.struct \n
.endm

.macro parm_sym n,rest:vararg
\n=.
\rest
.endm

.macro parm_end
ret_sz=.-parm_start
 pop_sect
.endm

.macro lstruct n=0
lstruct=\n
.struct \n
.endm

.macro lsym n,rest:vararg
\rest
\n=.
\n=-1*\n
.endm

.macro lstruct_end
lstruct_sz=.-lstruct
pop_sect
.endm

.macro _lstruct_end
lstruct_sz=.-lstruct
pop_sect
_enter $lstruct_sz
.endm

.ifndef win32
.ifndef _d_lnx
.include "dos-mac.h"
.endif
.endif

.macro movseg from,to,sz=
  push\sz \from
  pop\sz \to
.endm

.macro movseg_offs sega,offsa,segb,offsb
  movseg \sega,\segb
  mov  \offsa,\offsb
 .endm 

.macro bios_set_video_mode
  xor %ah,%ah
  int $0x10
.endm

.macro bios_get_rtc
  mov $2,%ah
  int $0x1a
.endm

.macro bios_get_cursor_pos
   mov $3,%ah
  int $0x10
.endm

.macro bios_tty_putc ch,pg=$0
  mov \pg,%bl
.ifnb \ch
  mov \ch,%al
.endif
  mov $0xe,%ah
   int $0x10
.endm

.macro bios_tty_write
  mov $0x13,%ah
   int $0x10
.endm

.macro init_serial_port
  xor %ah,%ah
  int $0x14
.endm

.macro write_serial_port
   mov $1,%ah
   int $0x14
.endm

.macro   read_serial_port
   mov $0x200,%ax
   int $0x14
.endm

.macro get_serial_port_status
   mov $0x300,%ax
   int $0x14
.endm

.macro bios_read_key
  xor %ah,%ah
  int $0x16
.endm

.macro bios_store_key
  mov $0x5,%ah
  int $0x16
.endm

.macro bios_read_keyx
  mov $0x10,%ah
  int $0x16
.endm

.macro bios_check_keyx
  mov $0x11,%ah
  int $0x16
.endm

.macro bios_check_keystroke
#  z=no key yet
#  nz=key pressed
  mov $1,%ah
  int $0x16
.endm

.macro disc_reset
  xor %ah,%ah
  int $0x13
.endm

.macro read_sector
  mov $_d_int13_read,%ah
  int $0x13
.endm

.macro write_sector n
.ifnb \n
  mov \n,%al
.endif
  mov $_d_int13_write,%ah
  int $0x13
.endm

.macro get_drive_parameters
 mov $8,%ah
  int $0x13
.endm

.macro ibm_ext_check
  mov $0x41,%ah
   mov $0x55aa,%bx
  int $0x13
.endm

.macro extended_read
  mov $0x42,%ah
  int $0x13
.endm


.macro ext_drive_parms
  mov $0x48,%ah
  int $0x13
.endm

.macro bios_ext_mem
  mov $0x88,%ah
	int $0x15
.endm

.macro switch_protected
  mov $0x89,%ah
  int $0x15
.endm

.macro bios_phoenix_ext_mem
  mov $0xe801,%ax
  int $0x15
.endm

.macro bios_disable_a20
   mov $0x2400,%ax
  int $0x15
.endm

.macro bios_enable_a20
   mov $0x2401,%ax
  int $0x15
.endm

.macro get_system_time
  xor %ah,%ah
  int $0x1a
.endm

.macro pci_bios_installation_check
  xor %edi,%edi
     mov $0xb101,%ax
   int $0x1a
.endm

.macro xpci_bios_installation_check
  pci_bios_installation_check
  exit_if_cf $1,"pci bios 2.0 or later not installed"
.endm

.macro pci_bios_find_device
  mov $0xb102,%ax
   int $0x1a
.endm

.macro pcibios_find_class cls,ix
.ifnb \ix
   setreg \ix,%si
.endif
.ifnb \cls
  mov \cls,%ecx
.endif
  mov $0xb103,%ax
   int $0x1a
.endm

.macro pci_find_device32
  mov $0xb182,%ax
   int $0x1a
.endm

.macro pci_bios_cfg_char
  mov $0xb108,%ax
  int $0x1a
.endm

.macro pci_bios_cfg_word
  mov $0xb109,%ax
  int $0x1a
.endm

.macro pci_bios_cfg_dword
  mov $0xb10a,%ax
  int $0x1a
.endm

.macro pci_bios_write_byte
  mov $0xb10b,%ax
  int $0x1a
.endm

.macro pci_bios_write_word
  mov $0xb10c,%ax
  int $0x1a
.endm

.macro pci_bios_write_dword
  mov $0xb10d,%ax
  int $0x1a
.endm

.macro dos_absolute_disc_read
  int $0x25
.endm

.macro dpmi_alloc_ldt n
.ifnb \n
  mov \n,%cx
.endif 
   xor %ax,%ax
   int $0x31
.endm

.macro dpmi_get_base_addr
   mov $6,%ax
  int $0x31
.endm

.macro dpmi_set_base_addr 
  mov $7,%ax
   int $0x31
.endm

.macro dpmi_set_seg_limit 
  mov $8,%ax
   int $0x31
.endm

.macro dpmi_set_access_rights 
  mov $9,%ax
   int $0x31
.endm

.macro dpmi_get_descriptor
  mov $0xb,%ax
  int $0x31
.endm

.macro dpmi_set_descriptor sel,desc
.ifnb \sel
  mov \sel,%bx
.endif
.ifnb \desc
  mov \desc,_di
.endif
  mov $0xc,%ax
  int $0x31
.endm

.macro dpmi_alloc_dos_memory_block
  mov $0x100,%ax
  int $0x31
.endm

.macro dpmi_free_dos_memory_block
  mov $0x101,%ax
  int $0x31
.endm

.macro dpmi_set_vector
  mov $0x205,%ax
  int $0x31
.endm

.macro dpmi_real_mode_interrupt
  mov $0x300,%ax
  int $0x31
.endm

.macro dpmi_alloc_memory_block
  mov $0x501,%ax
  int $0x31
.endm

.macro print str
  push _si
 mov $print_l\@,_si
   call32 dos_print
  pop _si
.data
print_l\@: .asciz "\str"
.text
.endm

.macro saveregs 
  pusha
  _pushregs ds,es,ss
   pushf
.endm

.macro restore_regs 
  popf
  _popregs ss,es,ds
  popa
.endm

.macro use_saved_reg_struct 
.struct 0
saved_flags: .word 0
saved_ss: .word 0
saved_es: .word 0
saved_ds: .word 0
saved_di: .word 0
saved_si: .word 0
saved_bp: .word 0
 saved_sp: .word 0
saved_bx: .word 0
saved_dx: .word 0
saved_cx: .word 0
saved_ax: .word 0
.endm 

.macro ifinrange ch,start,end,to
  cmp \start,\ch
  jb l\@
 cmp \end,\ch
  ja l\@ 
  jmp \to
l\@:
.endm

.macro ifnotinrange ch,start,end,to
  cmp \start,\ch
  jb \to 
 cmp \end,\ch
  ja \to 
.endm
.macro ifisspace to
  cmp $' ',%al
  jbe \to
  cmp $'z',%al
  ja \to
.endm

.macro ifiseol goto
   cmp $0xd,%al
  je \goto
  cmp $0xa,%al
  je \goto
.endm

.macro ifnalnum data,to
  cmpb $'0',\data
  jb \to
  cmpb $'z',\data
  ja \to
.endm

.macro skipspace start,len,error
l\@: mov %si,%ax
  sub \start,%al
    cmp \len,%al
  jge \error 
    lodsb
  ifisspace l\@ 
.endm

.macro if thing1,cmp,thing2,goto,suffix
  cmp\suffix \thing2,\thing1
  j\cmp \goto
.endm

.macro if_or thing1,thing2,cmp,goto,suffix
  or\suffix \thing1,\thing2
  j\cmp \goto
.endm

.macro if_rnz r,goto
  test \r,\r
  jnz \goto
.endm

.macro if_rz r,goto
  test \r,\r
  jz \goto
.endm

.macro if_z thing,goto,suffix
  test\suffix $-1,\thing
  jz \goto
.endm

.macro if_nz thing,goto,suffix
  if_or $0,\thing,nz,\goto,\suffix
.endm

.macro testz thing1,goto,suffix
  test\suffix $-1,\thing1
   jz \goto
.endm

.macro testnz thing1,goto,suffix
  test\suffix $-1,\thing1
   jnz \goto
.endm

.macro orif  thing,flag,goto
  or \thing,\thing
  j\flag \goto
.endm


.macro getip rc
  call l\@
l\@: pop \rc
  sub $3,\rc
.endm

.macro getstart_ip rc
  call l\@
l\@: pop \rc
  sub $l\@,\rc
.endm

.macro dos_newline
 dos_writedl_con $'\n'
.endm

.macro dt_set_port
  xor %dx,%dx
.endm

.macro dt_init
   dt_set_port
  mov $0xe3,%al # 9600 8 n 1
  init_serial_port
.endm

.macro if_dt_ok goto
  test $0x80,%ah
 jz \goto # timeuot not set
  puts "No double talk found"
    terminate 02
.endm

.macro dt_puts
  dt_set_port
  call bios_int14_puts
.endm

.macro dt_reset
  mov $_d_dt_flush_buffer_char,%al
   write_serial_port
.endm

.macro waitkey key,eq=ne
l\@:
          IN $0x60,%AL # Key pressed !
          CMP \key,%al
  j\eq l\@# was not our key, just go back to sleep.
.endm

.macro hwaitkey key,eq=ne
l\@:
          HLT        # suspend CPU internal bus clock
          IN $0x60,%AL # Key pressed !
          CMP \key,%al
  j\eq l\@# was not our key, just go back to sleep.
.endm

.macro countargs a=eof,b:vararg
.if \a!=eof
   lcount=lcount+1
  countargs \b
.else
lcount=lcount-1
.endif
.endm

.macro mklabels r=eof,r1:vararg
.if \r!=eof
l_\r=lcount*sizeof_int
   lcount=lcount-1
  mklabels \r1
.endif
.endm

.macro _pushregs r0=0,r1:vararg
.if \r0!=0
  push %\r0
    _pushregs \r1
.endif
.endm

.macro pushregs r0:vararg
lcount=0
countargs \r0
mklabels \r0
  _pushregs \r0
.endm

.macro _popregs r0=0,r1:vararg
.if \r0!=0
  pop %\r0
  _popregs \r1
.endif
.endm

.macro xpushregs r0=0,r1:vararg
.if \r0!=0
  push \r0
    xpushregs \r1
.endif
.endm

.macro xpushl r0=0,r1:vararg
.if \r0!=0
  pushl \r0
    xpushl \r1
.endif
.endm

.macro xpopregs r0=0,r1:vararg
.if \r0!=0
  pop \r0
  xpopregs \r1
.endif
.endm

.macro segoffs2linear seg,offs
   xor %eax,%eax
  mov \seg,%ax
   addl \offs,%eax
.endm


.macro cutmode_section f,o
.ifnb \o
.=0x\o
.endif
gsym cutmode_f_\f
.endm

.macro set_cutmode onoff
.global cutmode_\onoff\@
cutmode_\onoff\@=cut_val
.endm

.macro toggle_cutmode here,ch=0x90
.ifnb \here
t\here:
.org 0x\here,\ch
.endif
.ifndef cutmode
cutmode=0
.endif
   cut_val=.
cutmode=!cutmode
.if cutmode
set_cutmode on
.else
  set_cutmode off
.endif
.endm

.macro gsym lab,rest:vararg
.global \lab
\lab: \rest
.endm
.macro pic_gsym lab,rest:vararg
.type \lab,@function
  gsym \lab,\rest
.endm


.macro cgsym lab,rest:vararg
.global _\lab
_\lab: \rest
.endm


.macro playnote_ p,d
  mov \p,_ax
   mov \d,_cx
   call32 playnote_
.endm

.macro playnote_rtc p,d
  mov \p,%eax
   mov \d,%ecx
   call playnote_rtc
.endm

.macro playnote_ms p,d
  pushl \d
  pushl \p
   call32 playnote_ms
.endm

.macro playnote_ms32 p,d
  push \d
  push \p
   call playnote_ms32
.endm

.macro playnote_n32 p,d
  mov \p,%eax
   mov \d,%ecx
   call playnote_n32
.endm

.macro playnote_rtc32 p,d
  mov \p,%eax
   mov \d,%ecx
   call playnote_rtc32
.endm

.macro dos_playnote_ms p,d
  pushl \d
  push \p
   call dos_playnote_ms
.endm

.macro playnote_n p,d
  mov \p,%eax
   mov \d,%ecx
   call playnote_n
.endm

.macro _term rc
  mov $\rc,%al
   jmp _term
 .endm

.macro wait_forever
l\@: nop
  jmp l\@
.endm

.macro set_desc d,r=%eax
  mov \r,%ebx
  mov %bx,\d+2
  shr $16,%ebx
   mov %bl,\d+4
   mov %bh,\d+7
.endm

.macro movf d,w=
  pushf\w
  pop\w \d
.endm

.macro setf d,w=
  push\w \d
  popf\w
.endm

.macro  check_apic
   mov $1,%eax
  cpuid
  shr $7,%edx
   and $7,%edx
.endm

.macro desc_start l
.align 16
\l:
gdt_start=.
.space 8
.endm

.macro desc_entry l1,rest:vararg
\l1=.-gdt_start
 .word \rest
.endm

.macro tbl_entry tbl,nm,stuff:vararg
\nm=.-\tbl
gsym \tbl\nm .word \stuff
.endm

.macro gdt_entry base,limit,flags
.word \limit&0xffff
 .word \base&0xffff
  .byte (\base>>16)&0xff
  .byte \flags&0xff
  .byte (\limit>>16&0xf)|((\flags>>4)&0xf0)
  .byte \base>>0x18
.endm
.macro zeroreg r0=0,r1:vararg
.if \r0!=0
  xor \r0,\r0
    zeroreg \r1
.endif
.endm


.macro dt_flush
  dt_set_port
   xor %al,%al
   write_serial_port
.endm


.macro sys_iopl n
  mov \n,%ebx
   mov $110,%eax
  int $0x80
.endm

.macro sys_hack n
  mov \n,%ebx
   mov $285,%eax
  int $0x80
.endm

.macro set_if_cf r=%eax,l=l
  jnc l\@
  mov\l $-1,\r
l\@:
.endm

.macro ctr0_ticks t
  pushl \t
  call32  ctr0_ticks
.endm

.macro usleep n
  pushl \n
   call32 usleep
.endm

.macro flush_prefetch
  jmp  prefetch\@
prefetch\@:
.endm

.macro disable_nmi
  mov $0x80,%al
   out %al,$0x70
.endm

.macro enable_nmi
   zeroreg %al
  out %al,$0x70
  in $0x71,%al
.endm

.macro sound p
   mov \p,%cx
  mov $0x12,%dx
  mov $0x3456,%ax
   div %cx
  mov %ax,%dx
   mov $0xb6,%al
  out %al,$0x43
  mov %dx,%ax
 out %al,$0x42
  mov %ah,%al
  out %al,$0x42
  in $0x61,%al
  or $3,%al
  out %al,$0x61
.endm

.macro nosound
 in $0x61,%al
   and $0xfc,%al
  out %al,$0x61
.endm

.macro set_slave_pic mask
  mov \mask,%al
  out %al,$0xa1
  iodelay
.endm

.macro set_master_pic mask
  mov \mask,%al
  out %al,$0x21
  iodelay
.endm

# this is the standard way of doing a delay after
# any io op in linux so assume it necessary
# dont think the value in al is significant
.macro  iodelay
  out %al,$0x80
.endm


.macro _set_segs val
  mov \val,%eax
   mov %ax,%ds
  mov %ax,%es
   mov %ax,%ss
.endm

.macro set_segs val
  _set_segs \val
   mov %ax,%fs
   mov %ax,%gs
.endm

.macro set_counter0 a,val
   mov \a,%al
   out %al,$0x43
  mov \val,%ax
     out %al,$0x40
    mov %ah,%al
    out %al,$0x40
.endm

.macro xset_counter qq,rr
  pushw %ax
  set_counter0 \qq,\rr
  popw %ax
.endm

.macro hex2asc b,n
   pushl \n
  push  \b
  call32 hex2asc
.endm

.include "getopt-mac.h"

.macro if_set a,b,goto,sz
  test\sz \a,\b
  jnz \goto
.endm

.macro if_unset a,b,goto,sz
  test\sz \a,\b
  jz \goto
.endm

.macro pop_sect
.ifdef win32
.text
.exitm
.endif
.ifndef __djgpp__
.previous
.else
.text
.endif
.endm

.macro chs_read_sector d,c,h,s,seg,b,n
  xpushregs \n,\seg,\b
  call make_linear
  push \s
  push \h
  push \c
  push \d
  push $_d_int13_read
  call chs_rw_sector
.endm
  
.macro i2asc l,n
  push\l \n
  call i2asc\l
.endm

.macro serial_config p,d
  push \d
  push \p
  call32 serial_config
.endm

.macro serial_mcr p,v
  push \v
  push \p
  call32 serial_mcr
.endm

.macro serial_getch p,t
  pushl \t
   pushw \p
  call serial_getch
.endm

.macro serial_mputc a,u,tmo
  pushl \tmo
 push  \u
  push \a
  call32 serial_mputc
.endm

.macro serial_putc ch,p
  _push \p
    _push \ch
  call32 serial_putc
.endm

.macro parport_putc ch,p
  _push \p
    _push \ch
  call32 parport_putc
.endm

.macro serial_status p
  push \p
  call serial_status
.endm


.macro serial_puts0 str
   serial_puts $0x3f8,"\str"
.endm

.macro serial_puts p,s
   push $l\@
    push \p
  call32 serial_puts
.data
l\@: .asciz "\s"
pop_sect
.endm

.macro serial_puts32 p,s
   push $l\@
    push \p
  call serial_puts32
.data
l\@: .asciz "\s"
pop_sect
.endm

.macro port2uart p
  pushw \p
  call port2uart
.endm

.macro get_vector i
  pushw \i
  call get_vector
.endm

.macro set_vector i,seg,offs=none
.if \offs==none
  pushl \seg
.else
  pushw \seg
  pushw \offs
.endif
  pushw \i
 call set_vector
.endm

.macro estruct
ret_sz=.-2*sizeof_int
pop_sect
.endm
.macro pestruct
  ret_sz=.-(sizeof_int+s_pushax_sz)
  pop_sect
.endm

.macro lba_read d,sec,n,seg,offs
.ifb \offs
  pushl \seg
.else
  push \seg
   push \offs
  call make_linear
.endif
  push \n
   pushl \sec
    push \d
  call lba_read
.endm

.macro lba_write d,sec,n,seg,offs
.ifb \offs
  pushl \seg
.else
  push \seg
   push \offs
  call make_linear
.endif
  push \n
   pushl \sec
    push \d
  call lba_write
.endm

.macro stringz l,s
l: int_t l\@
.data 9999
l\@: .asciz "\s"
.previous
.endm

.macro read_mbr d,b
  pushl \b
  push \d
  call read_mbr
.endm

.macro read_part_offs_len d,p
   push \p
  push \d
  call32 read_part_offs_len
.endm

.macro _ret sz
.ifdef real_mode
  ret \sz
.else
  ret
.endif
.endm

.ifndef m4gs
.macro m4gs l,stuff:vararg
.global \l
\l: \stuff
.endm
.endif

.macro udelayx w,p
   pushl \p
   pushl \w
  call32 udelayx
.endm

.macro segsum i,n,s
  mov \s,%ax
  \i \n,%ax
  mov %ax,\s
.endm


.macro if_rt r,tst,goto
   test \r,\r
  j\tst \goto
.endm

.macro farcall seg,offs
  push %cs
  push $here\@
  flip_cs \seg,\offs
here\@:
.endm

# the to arg is the cs:offs paring
.macro pnpcall to
  push %cs
  push $l\@
  pushl \to
   lret
l\@:
.endm

.include "mactypes.h"
.ifdef _d_lnx
.include "linux-syscalls-mac.h"
.endif

.macro linelen p1
  push \p1
  call32 linelen
.endm

.macro row_col2ptr b,r,c
  push \c
  push \r
  push \b
  call32 row_col2ptr
.endm

.macro f_strcpy dest,src
  push \src
  push \dest
  call32 f_strcpy
.endm

.macro linestart l,p
  push \p
   push \l
   call32 linestart
.endm


.macro delete_line b,p
  push \p
  push \b
  call32 delete_line
.endm

.macro ved_strpcrlf b,l
   push \l
  push \b
  call32 ved_strpcrlf
.endm

.macro ved_delete_line b,lenp,cpos,rowp,colp
  push \colp
  push \rowp
   push \cpos
   push \lenp
  push \b
  call32 ved_delete_line
.endm

.macro is_cts port
  push \port
  call32 is_cts
.endm

.macro udelay n
  push \n
  call32 udelay
.endm


.macro udelay2 n
  push \n
  call32 udelay2
.endm

.macro _insb p,b,l
  push \l
 push \b
   push \p
  call32 insb
.endm

.macro _outsb p,b,l
  push \l
 push \b
   push \p
  call32 outsb
.endm

.macro _inb p,offs
.ifnb \offs
  push \offs
.endif
  push \p
.ifnb \offs
call32 ind_inb
.else
  call32 inb
.endif
.endm

.macro _inw p,o
.ifnb \o
   ind_inw \p,\o
.else
  push \p  
call32 inw
.endif
.endm

.macro _inl p,o
.ifnb \o
   ind_inl \p,\o
.else
  push \p  
call32 inl
.endif
.endm

.macro _outb v,p,offs
.ifnb \offs
  push \offs
.endif
   push \p
  push \v
.ifnb \offs
   call32 ind_outb
.else
  call32 outb
.endif
.endm

.macro _outw v,p,offs
.ifnb \offs
  push \offs
.endif
   push \p
  push \v
.ifnb \offs
  call32 ind_outw
.else
  call32 outw
.endif
.endm

.macro _outl v,p,offs
.ifnb \offs
  push \offs
.endif
   push \p
  pushl \v
.ifnb \offs
  call32 ind_outl
.else
  call32 outl
.endif
.endm

.macro ind_inb a,o
  push \o
   push \a
  call32 ind_inb
.endm

.macro ind_inw a,o
  push \o
   push \a
  call32 ind_inw
.endm

.macro ind_inl a,o
  push \o
   push \a
  call32 ind_inl
.endm

.macro ind_outb v,a,o
  push \o
  push \a
  push \v
  call32 ind_outb
.endm

.macro ind_outw v,a,o
  push \o
  push \a
  push \v
  call32 ind_outw
.endm

.macro ind_outl v,a,o
  push \o
  push \a
  pushl \v
  call32 ind_outl
.endm

# if no stack required
# push and mov takes less space, leave still works
.macro _enter n
.ifb \n
   push _bp
  mov _sp,_bp
.else
   enter \n,$0
.endif
.endm

.macro prot16_cs seg,desc
  push \desc
  push \seg
  call32 prot16_cs
.endm

.macro prot16_ds seg,desc
  push \desc
  push \seg
  call32 prot16_ds
.endm


.macro go_real sel
  push \sel
  call go_real
.endm


.macro go_protect seg,g,i
 push \i
  push \g
  push \seg
  call  go_protect
.endm


.macro bitmask a
  push \a
   call32 bitmask
.endm


.macro  get_bit_array array,b,pos
  push \pos
  push \b
   push \array
  call32 get_bit_array
.endm

.macro put_bit_array array,b,pos,v
  pushl \v
  push \pos
  push \b
   push \array
  call32 put_bit_array
.endm

.macro putcmos p,v
  push \v
  push \p
  call putcmos
.endm

.macro getcmos p
  push \p
  call getcmos
.endm


.macro bcd2word p
  push \p
   call32 bcd2word
.endm

.macro bcd2char p
  push \p
  call32 bcd2char
.endm
.macro get_days_in_month y,m
  push \m
   push \y
  call32 get_days_in_month
.endm

.macro bcd2days n
  pushl \n
  call32 bcd2days
.endm

.macro bcd2secs n
  pushl \n
  call32 bcd2secs
.endm


.macro serial_get_config u
  push \u
  call32 serial_get_config
.endm

.macro xget_drive_parms d,pb
  push \pb
  push \d
  call32 xget_drive_parms
.endm


.macro strchr a,b
   push \b
  push \a
  call32 strchr
.endm

.macro add_seg_offs n,s,offs
  push %eax
    mov %\s,%eax
  shl $4,%eax
   add %e\offs,%eax
    add \n,%eax
  mov %ax,%\offs
   shr $0x10,%eax
   shl $0xc,%ax
    mov %ax,%\s
  pop %eax
.endm

.macro linear2seg_offs n,s,o
  push %eax
  mov \n,%eax
  rol $0x10,%eax
  shl $0xc,%ax
   push %eax
  popw \s
  popw \o
  pop %eax
.endm


.macro add2 r
  inc \r
   inc  \r
.endm

.macro sub2 r
  dec \r
   dec  \r
.endm

.macro f_stristr h,n
  push \n
   push \h
  call32 f_stristr
.endm

.macro f_strstr h,n
  push \n
  push \h
  call32 f_strstr
.endm


.macro align_to a,b
  push \b
  push \a
  call align_to
.endm

.macro do_align a,b
  push \b
  push \a
  call do_align
.endm

.macro f_memset b,ch,n
  push \n
  push \ch
  push \b
  call32 f_memset
.endm

.macro f_memsetw b,ch,n
  push \n
  push \ch
  push \b
  call32 f_memsetw
.endm

.macro f_memsetl b,ch,n
  push \n
  pushl \ch
  push \b
  call32 f_memsetl
.endm

.macro make_linear s,b,z
   push \s
  push \b
  call make_linear
.ifnb \z
  popl \z
.endif
.endm

.macro ascin str
  .ascii "\str"
   .byte '\n'
.endm


.macro seg_offs2linear s,o,r
.ifb \o
  pushl \s
.else
  push \s
  push  \o
.endif
  call seg_offs2linear
  popl \r
.endm


.macro setreg v,r
  .if \v==$0
  zeroreg \r
.else
  mov \v,\r
.endif
.endm

.macro strtab a,b:vararg
   int_t l\@
.data 999
l\@: .asciz "\a"
.previous
.ifnb \b
   strtab \b
.endif
.endm


.macro parport_set_mode p,ch
  push \ch
  push \p
  call32 parport_set_mode
.endm

.macro parport_set_bi_direc p,ch
  push \ch
  push \p
  call32 parport_set_bi_direc
.endm

.macro divisor2baud d
  push \d
  call32 divisor2baud
.endm

.macro baud2divisor b
  pushl \b
  call32 baud2divisor
.endm

.macro set_com_parms port,b,cr,set_rts
  push \set_rts
  push \cr
   pushl \b
 push \port
   call32 set_com_parms
.endm


.macro uhci_check_cfg port
  push \port
  call32 uhci_check_cfg
.endm

.macro uhci_connect_change port
  push \port
  call32 uhci_connect_change
.endm

.macro incbin_till n,s,e,o=0,pad_ch=0
s\@=\s-\o
.org s\@,\pad_ch
.ifnb \e
l\@=\e-\s
.incbin "\n",s\@,l\@
.else
.incbin "\n",s\@
.endif
.endm

.macro mk_lab l
l\l=0x\l
.endm

.macro extmem_read_dword a,o=$0
push \o
  pushl \a
  call extmem_read_dword
.endm

.macro extmem_write_dword a,o=$0,v
  pushl \v
  push \o
  pushl \a
  call extmem_write_dword
.endm

.macro f_memcpy d,s,n
   push \n
  push \s
  push \d
  call f_memcpy
.endm

.macro align32_to a,b
  pushl \b
  pushl \a
  call align32_to
.endm

.macro extmem_setbits b,o,v
    pushl \v
   push \o
   pushl \b
  call32 extmem_setbits
.endm

.macro extmem_unsetbits b,o,v
    pushl \v
   push \o
   pushl \b
  call32 extmem_unsetbits
.endm

.macro pcicfg_read_byte a,b
  push \b
  push \a
  call32 pcicfg_read_byte
.endm

.macro pcicfg_read_word b,p
  push \p
  push \b
 call32 pcicfg_read_word
.endm

.macro pcicfg_read_dword sl,p
   push \p
   push \sl
  call32 pcicfg_read_dword
.endm
.macro pcicfg_write_byte a,b,c
  push \c
  push \b
  push \a
  call32 pcicfg_write_byte
.endm

.macro pcicfg_write_dword sl,p,v
  pushl \v
   push \p
   push \sl
  call32 pcicfg_write_dword
.endm

.macro io_setbitsw p,o,v
   push \v
   push \o
   push \p
  call32 io_setbitsw
.endm

.macro io_unsetbitsw p,o,v
   push \v
   push \o
   push \p
  call32 io_unsetbitsw
.endm

.macro do_urb h,u,pg0,sz,e
   push \e
  push \sz
   push \pg0
  push \u
  push \h
   call32 do_urb
.endm

.macro usb_get_string_descriptor h,ix,b,l,endpt
  push \endpt
  push \l
  push \b
   push \ix
   push \h
  call32 usb_get_string_descriptor
.endm

.macro usb_get_descriptor h,v,ix,b,l,endpt
  push \endpt
   push \l
  push \b
.ifb \ix
  pushl \v
.else
   pushw \ix
   pushw \v
.endif
   push \h
  call32 usb_get_descriptor
.endm

.macro usb_get_dev_descriptor h,ix,b,endpt
  push \endpt
  push \b
   push \ix
   push \h
  call32 usb_get_dev_descriptor
.endm

.macro usb_set_dev_addr h,a
  push \a
   push \h
  call32 usb_set_dev_addr
.endm

.macro days_in_year y
  push \y
  call32 days_in_year
.endm

.macro char2bcd n
  push \n
  call32 char2bcd
.endm
.macro word2bcd n
  push \n
  call32 word2bcd
.endm


.macro unix2bcd y,b
  push \b
  pushl \y
  call32 unix2bcd
.endm


.macro bcd2fatdate y
  push \y
  call32 bcd2fatdate
.endm

.macro bcd2fattime y
  push \y
  call32 bcd2fattime
.endm

.macro setbits_cr0 n
   mov %cr0,%eax
    or \n,%al
    mov %eax,%cr0
.endm

.macro andbits_cr0 n
   mov %cr0,%eax
  and  \n,%al
    mov %eax,%cr0
.endm

.macro _string_n b,n
  push \n
  push \b
  call32 _string_n
.endm

.macro sblast_spkr io,on_off
  push \on_off
  push \io
  call32 sblast_spkr
.endm
.macro  sblast_getch port
  push \port
  call32 sblast_getch
.endm

.macro sblast_putc b,p
  push \p
  push \b
 call32 sblast_putc
.endm

.macro sblast_dio_putc b,p
  push \p
  push \b
 call32 sblast_dio_putc
.endm

.macro enable_irq irq
  push \irq
  call32 enable_irq
.endm

.macro disable_irq irq
  push \irq
  call32 disable_irq
.endm

.macro io_addcrlf ob,ib,n,lc
  push \lc
  push \n
  push \ib
   push \ob
  call32 io_addcrlf
.endm

.macro set_irq p,n,on_off
  push \on_off
  push \n
  push \p
  call32 set_irq
.endm

.macro sblast_reset p
   push  \p
  call sblast_reset
.endm

.macro _penter
  pusha
  mov _sp,_bp
.endm

.macro _pleave
  mov _bp,_sp
  popa
.endm

.macro set_intr_gate  no,idt,sel,seg,addr
.ifb \addr
  pushl \seg
.else
  push \seg
  push \addr
  call make_linear
.endif
  push \sel
  push \idt
  push \no
  call32 set_intr_gate
.endm

.macro f_bios_tty_putc ch
  push \ch
 call f_bios_tty_putc
.endm

.macro f_bios_write str,len
  push \len
  push \str
  call f_bios_tty_write
.endm

.macro f_bios_tty_print str
  push \str
  call  f_bios_tty_print
.endm
.macro addcrlf_insitu buf,sz 
  push \sz
  push \buf
  call addcrlf_insitu
.endm
.macro double_fault
  mov %cr0,%eax
   or $1,%al
  mov %eax,%cr0
  ljmp $0,$0
.endm
.macro set_errno
  pusha
  mov %eax,%ebx
   call __errno_location
   mov %ebx,(%eax)
  popa
.endm

.endif
